package com.github.polpetta.mezzotre.orm.model;

import com.github.polpetta.types.json.CallbackQueryMetadata;
import io.ebean.annotation.DbJson;
import io.ebean.annotation.Length;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

// Could be better to adopt redis? Seems too much overhead for now...

/**
 * This entity allows for the push and retrieval of {@link CallbackQueryMetadata} related to a
 * particular <i>InlineKeyboardButton</i>. {@link CallbackQueryMetadata} is a loosely-structured
 * object, and this is by design: the fields stored in it depends on the metadata of the callback
 * and on the type of event
 *
 * @author Davide Polonio
 * @since 1.0
 */
@Entity
@Table(name = "callback_query_context")
public class CallbackQueryContext extends Base {

  @Id
  @Length(36)
  private final String id;

  @NotNull
  @Length(36)
  private final String entryGroup;

  @DbJson
  @Column(columnDefinition = "json not null default '{}'::json")
  @NotNull
  private final CallbackQueryMetadata fields;

  public CallbackQueryContext(String id, String entryGroup, CallbackQueryMetadata fields) {
    this.id = id;
    this.entryGroup = entryGroup;
    this.fields = fields;
  }

  public String getId() {
    return id;
  }

  public String getEntryGroup() {
    return entryGroup;
  }

  public CallbackQueryMetadata getFields() {
    return fields;
  }

  @Override
  public String toString() {
    return "CallbackQueryContext{"
        + "id='"
        + id
        + '\''
        + ", entryGroup='"
        + entryGroup
        + '\''
        + ", fields="
        + fields
        + '}';
  }
}
