package com.github.polpetta.mezzotre.telegram.callbackquery;

import com.github.polpetta.mezzotre.orm.model.CallbackQueryContext;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

@Singleton
public class Dispatcher {

  private final Set<Processor> tgEventProcessors;
  private final Executor threadPool;

  @Inject
  public Dispatcher(
      @Named("eventProcessors") Set<Processor> tgEventProcessors,
      @Named("eventThreadPool") Executor threadPool) {
    this.tgEventProcessors = tgEventProcessors;
    this.threadPool = threadPool;
  }

  public CompletableFuture<Optional<BaseRequest<?, ?>>> dispatch(
      CallbackQueryContext callbackQueryContext, Update update) {
    return CompletableFuture.completedFuture(update)
        .thenComposeAsync(
            ignored ->
                Optional.of(callbackQueryContext.getFields().getEvent())
                    .flatMap(
                        eventName ->
                            tgEventProcessors.stream()
                                .filter(processor -> processor.getEventName().equals(eventName))
                                .findAny())
                    .map(processor -> processor.process(callbackQueryContext, update))
                    .orElse(CompletableFuture.failedFuture(new EventProcessorNotFoundException())),
            threadPool);
  }
}
